@@ -21,6 +21,7 @@ from abc import ABCMeta, abstractmethod
 
 
 import numpy as np
+from scipy.special import logsumexp
 
 from sklearn.base import BaseEstimator, ClassifierMixin
 from sklearn.preprocessing import binarize
@@ -28,10 +29,10 @@ from sklearn.preprocessing import LabelBinarizer
 from sklearn.preprocessing import label_binarize
 from sklearn.utils import check_X_y, check_array, deprecated
 from sklearn.utils.extmath import safe_sparse_dot
-from sklearn.utils.fixes import logsumexp
 from sklearn.utils.multiclass import _check_partial_fit_first_call
 from sklearn.utils.validation import check_is_fitted, check_non_negative, column_or_1d
 from sklearn.utils.validation import _check_sample_weight
+from sklearn.utils.validation import _deprecate_positional_args
 import inaccel.coral as inaccel
 from sklearn.naive_bayes import GaussianNB as GaussianNBRef
 
@@ -47,7 +48,7 @@ class _BaseNB(ClassifierMixin, BaseEstimator, metaclass=ABCMeta):
         """Compute the unnormalized posterior log probability of X
 
         I.e. ``log P(c) + log P(x|c)`` for all rows x of X, as an array-like of
-        shape [n_classes, n_samples].
+        shape (n_classes, n_samples).
 
         Input is passed to _joint_log_likelihood as-is by predict,
         predict_proba and predict_log_proba.
@@ -133,35 +134,37 @@ class GaussianNB(_BaseNB):
 
     Parameters
     ----------
-    priors : array-like, shape (n_classes,)
+    priors : array-like of shape (n_classes,)
         Prior probabilities of the classes. If specified the priors are not
         adjusted according to the data.
 
-    var_smoothing : float, optional (default=1e-9)
+    var_smoothing : float, default=1e-9
         Portion of the largest variance of all features that is added to
         variances for calculation stability.
 
+        .. versionadded:: 0.20
+
     n_accel : int, default=8
         The number of accelerators to try to use for the computation.
 
     Attributes
     ----------
-    class_count_ : array, shape (n_classes,)
+    class_count_ : ndarray of shape (n_classes,)
         number of training samples observed in each class.
 
-    class_prior_ : array, shape (n_classes,)
+    class_prior_ : ndarray of shape (n_classes,)
         probability of each class.
 
-    classes_ : array, shape (n_classes,)
+    classes_ : ndarray of shape (n_classes,)
         class labels known to the classifier
 
     epsilon_ : float
         absolute additive value to variances
 
-    sigma_ : array, shape (n_classes, n_features)
+    sigma_ : ndarray of shape (n_classes, n_features)
         variance of each feature per class
 
-    theta_ : array, shape (n_classes, n_features)
+    theta_ : ndarray of shape (n_classes, n_features)
         mean of each feature per class
 
     Examples
@@ -182,7 +185,8 @@ class GaussianNB(_BaseNB):
     [1]
     """
 
-    def __init__(self, priors=None, var_smoothing=1e-9, n_accel=8):
+    @_deprecate_positional_args
+    def __init__(self, *, priors=None, var_smoothing=1e-9, n_accel=8):
         self.priors = priors
         self.var_smoothing = var_smoothing
         self.n_accel = n_accel
@@ -192,14 +196,14 @@ class GaussianNB(_BaseNB):
 
         Parameters
         ----------
-        X : array-like, shape (n_samples, n_features)
+        X : array-like of shape (n_samples, n_features)
             Training vectors, where n_samples is the number of samples
             and n_features is the number of features.
 
-        y : array-like, shape (n_samples,)
+        y : array-like of shape (n_samples,)
             Target values.
 
-        sample_weight : array-like, shape (n_samples,), optional (default=None)
+        sample_weight : array-like of shape (n_samples,), default=None
             Weights applied to individual samples (1. for unweighted).
 
             .. versionadded:: 0.17
@@ -209,6 +213,7 @@ class GaussianNB(_BaseNB):
         -------
         self : object
         """
+        X, y = self._validate_data(X, y)
         y = column_or_1d(y, warn=True)
         return self._partial_fit(X, y, np.unique(y), _refit=True,
                                  sample_weight=sample_weight)
@@ -239,21 +244,21 @@ class GaussianNB(_BaseNB):
             weights were given, this should contain the sum of sample
             weights represented in old mean and variance.
 
-        mu : array-like, shape (number of Gaussians,)
+        mu : array-like of shape (number of Gaussians,)
             Means for Gaussians in original set.
 
-        var : array-like, shape (number of Gaussians,)
+        var : array-like of shape (number of Gaussians,)
             Variances for Gaussians in original set.
 
-        sample_weight : array-like, shape (n_samples,), optional (default=None)
+        sample_weight : array-like of shape (n_samples,), default=None
             Weights applied to individual samples (1. for unweighted).
 
         Returns
         -------
-        total_mu : array-like, shape (number of Gaussians,)
+        total_mu : array-like of shape (number of Gaussians,)
             Updated mean for each Gaussian over the combined set.
 
-        total_var : array-like, shape (number of Gaussians,)
+        total_var : array-like of shape (number of Gaussians,)
             Updated variance for each Gaussian over the combined set.
         """
         if X.shape[0] == 0:
@@ -307,20 +312,20 @@ class GaussianNB(_BaseNB):
 
         Parameters
         ----------
-        X : array-like, shape (n_samples, n_features)
+        X : array-like of shape (n_samples, n_features)
             Training vectors, where n_samples is the number of samples and
             n_features is the number of features.
 
-        y : array-like, shape (n_samples,)
+        y : array-like of shape (n_samples,)
             Target values.
 
-        classes : array-like, shape (n_classes,), optional (default=None)
+        classes : array-like of shape (n_classes,), default=None
             List of all the classes that can possibly appear in the y vector.
 
             Must be provided at the first call to partial_fit, can be omitted
             in subsequent calls.
 
-        sample_weight : array-like, shape (n_samples,), optional (default=None)
+        sample_weight : array-like of shape (n_samples,), default=None
             Weights applied to individual samples (1. for unweighted).
 
             .. versionadded:: 0.17
@@ -338,24 +343,24 @@ class GaussianNB(_BaseNB):
 
         Parameters
         ----------
-        X : array-like, shape (n_samples, n_features)
+        X : array-like of shape (n_samples, n_features)
             Training vectors, where n_samples is the number of samples and
             n_features is the number of features.
 
-        y : array-like, shape (n_samples,)
+        y : array-like of shape (n_samples,)
             Target values.
 
-        classes : array-like, shape (n_classes,), optional (default=None)
+        classes : array-like of shape (n_classes,), default=None
             List of all the classes that can possibly appear in the y vector.
 
             Must be provided at the first call to partial_fit, can be omitted
             in subsequent calls.
 
-        _refit : bool, optional (default=False)
+        _refit : bool, default=False
             If true, act as though this were the first time we called
             _partial_fit (ie, throw away any past fitting and start over).
 
-        sample_weight : array-like, shape (n_samples,), optional (default=None)
+        sample_weight : array-like of shape (n_samples,), default=None
             Weights applied to individual samples (1. for unweighted).
 
         Returns
@@ -479,6 +484,12 @@ class GaussianNB(_BaseNB):
             X = self._check_X(X)
             n_samples, n_features = X.shape
             n_classes = np.size(self.classes_)
+            
+            assert n_features <= 2048, 'Unsupported n_features=%s. The '\
+                'accelerator supports maximum %s features.' % (n_features, 2048)
+            
+            assert n_classes <= 64, 'Unsupported n_classes=%s. The '\
+                'accelerator supports maximum %s classes.' % (n_classes, 64)
 
             #Data partitioning and allocating the division-remaining samples
             #to one of the chunks
@@ -562,7 +573,7 @@ class _BaseDiscreteNB(_BaseNB):
         return check_array(X, accept_sparse='csr')
 
     def _check_X_y(self, X, y):
-        return check_X_y(X, y, accept_sparse='csr')
+        return self._validate_data(X, y, accept_sparse='csr')
 
     def _update_class_log_prior(self, class_prior=None):
         n_classes = len(self.classes_)
@@ -621,7 +632,7 @@ class _BaseDiscreteNB(_BaseNB):
         y : array-like of shape (n_samples,)
             Target values.
 
-        classes : array-like of shape (n_classes) (default=None)
+        classes : array-like of shape (n_classes), default=None
             List of all the classes that can possibly appear in the y vector.
 
             Must be provided at the first call to partial_fit, can be omitted
@@ -659,8 +670,9 @@ class _BaseDiscreteNB(_BaseNB):
         # We convert it to np.float64 to support sample_weight consistently
         Y = Y.astype(np.float64, copy=False)
         if sample_weight is not None:
+            sample_weight = _check_sample_weight(sample_weight, X)
             sample_weight = np.atleast_2d(sample_weight)
-            Y *= check_array(sample_weight).T
+            Y *= sample_weight.T
 
         class_prior = self.class_prior
 
@@ -711,9 +723,9 @@ class _BaseDiscreteNB(_BaseNB):
         # this means we also don't have to cast X to floating point
         if sample_weight is not None:
             Y = Y.astype(np.float64, copy=False)
-            sample_weight = np.asarray(sample_weight)
+            sample_weight = _check_sample_weight(sample_weight, X)
             sample_weight = np.atleast_2d(sample_weight)
-            Y *= check_array(sample_weight).T
+            Y *= sample_weight.T
 
         class_prior = self.class_prior
 
@@ -763,44 +775,44 @@ class MultinomialNB(_BaseDiscreteNB):
 
     Parameters
     ----------
-    alpha : float, optional (default=1.0)
+    alpha : float, default=1.0
         Additive (Laplace/Lidstone) smoothing parameter
         (0 for no smoothing).
 
-    fit_prior : boolean, optional (default=True)
+    fit_prior : bool, default=True
         Whether to learn class prior probabilities or not.
         If false, a uniform prior will be used.
 
-    class_prior : array-like, size (n_classes,), optional (default=None)
+    class_prior : array-like of shape (n_classes,), default=None
         Prior probabilities of the classes. If specified the priors are not
         adjusted according to the data.
 
     Attributes
     ----------
-    class_count_ : array, shape (n_classes,)
+    class_count_ : ndarray of shape (n_classes,)
         Number of samples encountered for each class during fitting. This
         value is weighted by the sample weight when provided.
 
-    class_log_prior_ : array, shape (n_classes, )
+    class_log_prior_ : ndarray of shape (n_classes, )
         Smoothed empirical log probability for each class.
 
-    classes_ : array, shape (n_classes,)
+    classes_ : ndarray of shape (n_classes,)
         Class labels known to the classifier
 
-    coef_ : array, shape (n_classes, n_features)
+    coef_ : ndarray of shape (n_classes, n_features)
         Mirrors ``feature_log_prob_`` for interpreting MultinomialNB
         as a linear model.
 
-    feature_count_ : array, shape (n_classes, n_features)
+    feature_count_ : ndarray of shape (n_classes, n_features)
         Number of samples encountered for each (class, feature)
         during fitting. This value is weighted by the sample weight when
         provided.
 
-    feature_log_prob_ : array, shape (n_classes, n_features)
+    feature_log_prob_ : ndarray of shape (n_classes, n_features)
         Empirical log probability of features
         given a class, ``P(x_i|y)``.
 
-    intercept_ : array, shape (n_classes, )
+    intercept_ : ndarray of shape (n_classes, )
         Mirrors ``class_log_prior_`` for interpreting MultinomialNB
         as a linear model.
 
@@ -833,7 +845,8 @@ class MultinomialNB(_BaseDiscreteNB):
     https://nlp.stanford.edu/IR-book/html/htmledition/naive-bayes-text-classification-1.html
     """
 
-    def __init__(self, alpha=1.0, fit_prior=True, class_prior=None):
+    @_deprecate_positional_args
+    def __init__(self, *, alpha=1.0, fit_prior=True, class_prior=None):
         self.alpha = alpha
         self.fit_prior = fit_prior
         self.class_prior = class_prior
@@ -870,18 +883,20 @@ class ComplementNB(_BaseDiscreteNB):
 
     Read more in the :ref:`User Guide <complement_naive_bayes>`.
 
+    .. versionadded:: 0.20
+
     Parameters
     ----------
-    alpha : float, optional (default=1.0)
+    alpha : float, default=1.0
         Additive (Laplace/Lidstone) smoothing parameter (0 for no smoothing).
 
-    fit_prior : boolean, optional (default=True)
+    fit_prior : bool, default=True
         Only used in edge case with a single class in the training set.
 
-    class_prior : array-like, size (n_classes,), optional (default=None)
+    class_prior : array-like of shape (n_classes,), default=None
         Prior probabilities of the classes. Not used.
 
-    norm : boolean, optional (default=False)
+    norm : bool, default=False
         Whether or not a second normalization of the weights is performed. The
         default behavior mirrors the implementations found in Mahout and Weka,
         which do not follow the full algorithm described in Table 9 of the
@@ -889,26 +904,26 @@ class ComplementNB(_BaseDiscreteNB):
 
     Attributes
     ----------
-    class_count_ : array, shape (n_classes,)
+    class_count_ : ndarray of shape (n_classes,)
         Number of samples encountered for each class during fitting. This
         value is weighted by the sample weight when provided.
 
-    class_log_prior_ : array, shape (n_classes, )
+    class_log_prior_ : ndarray of shape (n_classes,)
         Smoothed empirical log probability for each class. Only used in edge
         case with a single class in the training set.
 
-    classes_ : array, shape (n_classes,)
+    classes_ : ndarray of shape (n_classes,)
         Class labels known to the classifier
 
-    feature_all_ : array, shape (n_features,)
+    feature_all_ : ndarray of shape (n_features,)
         Number of samples encountered for each feature during fitting. This
         value is weighted by the sample weight when provided.
 
-    feature_count_ : array, shape (n_classes, n_features)
+    feature_count_ : ndarray of shape (n_classes, n_features)
         Number of samples encountered for each (class, feature) during fitting.
         This value is weighted by the sample weight when provided.
 
-    feature_log_prob_ : array, shape (n_classes, n_features)
+    feature_log_prob_ : ndarray of shape (n_classes, n_features)
         Empirical weights for class complements.
 
     n_features_ : int
@@ -935,7 +950,8 @@ class ComplementNB(_BaseDiscreteNB):
     https://people.csail.mit.edu/jrennie/papers/icml03-nb.pdf
     """
 
-    def __init__(self, alpha=1.0, fit_prior=True, class_prior=None,
+    @_deprecate_positional_args
+    def __init__(self, *, alpha=1.0, fit_prior=True, class_prior=None,
                  norm=False):
         self.alpha = alpha
         self.fit_prior = fit_prior
@@ -983,46 +999,45 @@ class BernoulliNB(_BaseDiscreteNB):
 
     Parameters
     ----------
-    alpha : float, optional (default=1.0)
+    alpha : float, default=1.0
         Additive (Laplace/Lidstone) smoothing parameter
         (0 for no smoothing).
 
-    binarize : float or None, optional (default=0.0)
+    binarize : float or None, default=0.0
         Threshold for binarizing (mapping to booleans) of sample features.
         If None, input is presumed to already consist of binary vectors.
 
-    fit_prior : bool, optional (default=True)
+    fit_prior : bool, default=True
         Whether to learn class prior probabilities or not.
         If false, a uniform prior will be used.
 
-    class_prior : array-like, size=[n_classes,], optional (default=None)
+    class_prior : array-like of shape (n_classes,), default=None
         Prior probabilities of the classes. If specified the priors are not
         adjusted according to the data.
 
     Attributes
     ----------
-    class_count_ : array, shape = [n_classes]
+    class_count_ : ndarray of shape (n_classes)
         Number of samples encountered for each class during fitting. This
         value is weighted by the sample weight when provided.
 
-    class_log_prior_ : array, shape = [n_classes]
+    class_log_prior_ : ndarray of shape (n_classes)
         Log probability of each class (smoothed).
 
-    classes_ : array, shape (n_classes,)
+    classes_ : ndarray of shape (n_classes,)
         Class labels known to the classifier
 
-    feature_count_ : array, shape = [n_classes, n_features]
+    feature_count_ : ndarray of shape (n_classes, n_features)
         Number of samples encountered for each (class, feature)
         during fitting. This value is weighted by the sample weight when
         provided.
 
-    feature_log_prob_ : array, shape = [n_classes, n_features]
+    feature_log_prob_ : ndarray of shape (n_classes, n_features)
         Empirical log probability of features given a class, P(x_i|y).
 
     n_features_ : int
         Number of features of each sample.
 
-
     Examples
     --------
     >>> import numpy as np
@@ -1050,7 +1065,8 @@ class BernoulliNB(_BaseDiscreteNB):
     naive Bayes -- Which naive Bayes? 3rd Conf. on Email and Anti-Spam (CEAS).
     """
 
-    def __init__(self, alpha=1.0, binarize=.0, fit_prior=True,
+    @_deprecate_positional_args
+    def __init__(self, *, alpha=1.0, binarize=.0, fit_prior=True,
                  class_prior=None):
         self.alpha = alpha
         self.binarize = binarize
@@ -1110,36 +1126,36 @@ class CategoricalNB(_BaseDiscreteNB):
 
     Parameters
     ----------
-    alpha : float, optional (default=1.0)
+    alpha : float, default=1.0
         Additive (Laplace/Lidstone) smoothing parameter
         (0 for no smoothing).
 
-    fit_prior : boolean, optional (default=True)
+    fit_prior : bool, default=True
         Whether to learn class prior probabilities or not.
         If false, a uniform prior will be used.
 
-    class_prior : array-like, size (n_classes,), optional (default=None)
+    class_prior : array-like of shape (n_classes,), default=None
         Prior probabilities of the classes. If specified the priors are not
         adjusted according to the data.
 
     Attributes
     ----------
-    category_count_ : list of arrays, len n_features
+    category_count_ : list of arrays of shape (n_features,)
         Holds arrays of shape (n_classes, n_categories of respective feature)
         for each feature. Each array provides the number of samples
         encountered for each class and category of the specific feature.
 
-    class_count_ : array, shape (n_classes,)
+    class_count_ : ndarray of shape (n_classes,)
         Number of samples encountered for each class during fitting. This
         value is weighted by the sample weight when provided.
 
-    class_log_prior_ : array, shape (n_classes, )
+    class_log_prior_ : ndarray of shape (n_classes,)
         Smoothed empirical log probability for each class.
 
-    classes_ : array, shape (n_classes,)
+    classes_ : ndarray of shape (n_classes,)
         Class labels known to the classifier
 
-    feature_log_prob_ : list of arrays, len n_features
+    feature_log_prob_ : list of arrays of shape (n_features,)
         Holds arrays of shape (n_classes, n_categories of respective feature)
         for each feature. Each array provides the empirical log probability
         of categories given the respective feature and class, ``P(x_i|y)``.
@@ -1161,7 +1177,8 @@ class CategoricalNB(_BaseDiscreteNB):
     [3]
     """
 
-    def __init__(self, alpha=1.0, fit_prior=True, class_prior=None):
+    @_deprecate_positional_args
+    def __init__(self, *, alpha=1.0, fit_prior=True, class_prior=None):
         self.alpha = alpha
         self.fit_prior = fit_prior
         self.class_prior = class_prior
@@ -1171,7 +1188,7 @@ class CategoricalNB(_BaseDiscreteNB):
 
         Parameters
         ----------
-        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
+        X : {array-like, sparse matrix} of shape (n_samples, n_features)
             Training vectors, where n_samples is the number of samples and
             n_features is the number of features. Here, each feature of X is
             assumed to be from a different categorical distribution.
@@ -1180,10 +1197,10 @@ class CategoricalNB(_BaseDiscreteNB):
             total number of categories for the given feature. This can, for
             instance, be achieved with the help of OrdinalEncoder.
 
-        y : array-like, shape = [n_samples]
+        y : array-like of shape (n_samples,)
             Target values.
 
-        sample_weight : array-like, shape = [n_samples], (default=None)
+        sample_weight : array-like of shape (n_samples), default=None
             Weights applied to individual samples (1. for unweighted).
 
         Returns
@@ -1208,7 +1225,7 @@ class CategoricalNB(_BaseDiscreteNB):
 
         Parameters
         ----------
-        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
+        X : {array-like, sparse matrix} of shape (n_samples, n_features)
             Training vectors, where n_samples is the number of samples and
             n_features is the number of features. Here, each feature of X is
             assumed to be from a different categorical distribution.
@@ -1217,16 +1234,16 @@ class CategoricalNB(_BaseDiscreteNB):
             total number of categories for the given feature. This can, for
             instance, be achieved with the help of OrdinalEncoder.
 
-        y : array-like, shape = [n_samples]
+        y : array-like of shape (n_samples)
             Target values.
 
-        classes : array-like, shape = [n_classes] (default=None)
+        classes : array-like of shape (n_classes), default=None
             List of all the classes that can possibly appear in the y vector.
 
             Must be provided at the first call to partial_fit, can be omitted
             in subsequent calls.
 
-        sample_weight : array-like, shape = [n_samples], (default=None)
+        sample_weight : array-like of shape (n_samples), default=None
             Weights applied to individual samples (1. for unweighted).
 
         Returns
@@ -1236,24 +1253,19 @@ class CategoricalNB(_BaseDiscreteNB):
         return super().partial_fit(X, y, classes,
                                    sample_weight=sample_weight)
 
+    def _more_tags(self):
+        return {'requires_positive_X': True}
+
     def _check_X(self, X):
-        # FIXME: we can avoid calling check_array twice after #14872 is merged.
-        # X = check_array(X, y, dtype='int', accept_sparse=False,
-        #                 force_all_finite=True)
-        X = check_array(X, accept_sparse=False, force_all_finite=True)
-        X = check_array(X, dtype='int')
-        if np.any(X < 0):
-            raise ValueError("X must not contain negative values.")
+        X = check_array(X, dtype='int', accept_sparse=False,
+                        force_all_finite=True)
+        check_non_negative(X, "CategoricalNB (input X)")
         return X
 
     def _check_X_y(self, X, y):
-        # FIXME: we can avoid calling check_array twice after #14872 is merged.
-        # X, y = check_array(X, y, dtype='int', accept_sparse=False,
-        #                    force_all_finite=True)
-        X, y = check_X_y(X, y, accept_sparse=False, force_all_finite=True)
-        X, y = check_X_y(X, y, dtype='int')
-        if np.any(X < 0):
-            raise ValueError("X must not contain negative values.")
+        X, y = self._validate_data(X, y, dtype='int', accept_sparse=False,
+                                   force_all_finite=True)
+        check_non_negative(X, "CategoricalNB (input X)")
         return X, y
 
     def _init_counters(self, n_effective_classes, n_features):
@@ -1302,7 +1314,7 @@ class CategoricalNB(_BaseDiscreteNB):
     def _joint_log_likelihood(self, X):
         if not X.shape[1] == self.n_features_:
             raise ValueError("Expected input with %d features, got %d instead"
-                             .format(self.n_features_, X.shape[1]))
+                             % (self.n_features_, X.shape[1]))
         jll = np.zeros((X.shape[0], self.class_count_.shape[0]))
         for i in range(self.n_features_):
             indices = X[:, i]
